import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

class ButtonGroup extends StatefulWidget {
  const ButtonGroup({
    Key? key,
    required this.buttons,
    this.onTap,
    this.selectedIndex = -1,
    this.textColor = const Color(0xff969799),
    this.textSelectedColor = Colors.white,
    this.color = Colors.white,
    this.selectedColor = const Color(0xFF1989FA),
    this.borderColor = const Color(0xffF2F3F5),
    this.borderSelectColor = const Color(0xFF1989FA),
  }) : super(key: key);
  final List<String> buttons;
  final Color textColor;
  final Color textSelectedColor;
  final Color color;
  final Color selectedColor;
  final int selectedIndex;
  final Color borderColor;
  final Color borderSelectColor;
  final Function(String button, int index)? onTap;

  @override
  State<StatefulWidget> createState() {
    return _ButtonGroupState();
  }
}

class _ButtonGroupState extends State<ButtonGroup> {
  int _selectedIndex = -1;
  @override
  void initState() {
    _selectedIndex = widget.selectedIndex;
    super.initState();
  }

  @override
  void didUpdateWidget(ButtonGroup oldWidget) {
    if (oldWidget.selectedIndex != _selectedIndex) {
      _selectedIndex = widget.selectedIndex;
    }
    super.didUpdateWidget(oldWidget);
  }

  _renderSingleButton(String button, int index) {
    var bgColor = index == _selectedIndex ? widget.selectedColor : widget.color;
    var tColor = index == _selectedIndex ? widget.textSelectedColor : widget.textColor;
    var borderColor = index == _selectedIndex ? widget.borderSelectColor : widget.borderColor;
    var decoration = BoxDecoration(
        color: bgColor,
        border: Border.all(
          color: borderColor,
        ),
        borderRadius: BorderRadius.circular(4));

    return GestureDetector(
      onTap: () {
        setState(() {
          _selectedIndex = index;
        });
        if (widget.onTap != null) {
          widget.onTap!(widget.buttons[index], index);
        }
      },
      child: Container(
        constraints: BoxConstraints(
          minHeight: 28.h,
          maxWidth: 64.w,
        ),
        decoration: decoration,
        padding: const EdgeInsets.symmetric(vertical: 6.0, horizontal: 8.0),
        child: Center(
          child: Text(
            button,
            style: TextStyle(color: tColor, fontSize: 14.sp),
          ),
        ),
      ),
    );
  }

  _renderButtons() {
    var buttonWidgets = <Widget>[];
    for (var i = 0; i < widget.buttons.length; i++) {
      buttonWidgets.add(_renderSingleButton(widget.buttons[i], i));
    }
    return buttonWidgets;
  }

  @override
  Widget build(BuildContext context) {
    return Wrap(
      spacing: 4.w,
      runSpacing: 12.h,
      alignment: WrapAlignment.start,
      children: _renderButtons(),
    );
  }
}
