import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

enum TLWrapChoiceBoxType {
  none,
  center,
  wrap,
  wrapSpaceBetween,
}

class TLWrapChoiceBox<T> extends StatefulWidget {
  const TLWrapChoiceBox(
    this.title, {
    Key? key,
    this.items,
    this.titleTransformer,
    this.isSelected,
    this.onValueChange,
    //
    this.disabel = false,
    this.style,
    this.padding,
    this.spacing,
    this.runSpacing,
    this.type = TLWrapChoiceBoxType.none,
  }) : super(key: key);

  final String title;

  final List<T>? items;
  final String Function(T item)? titleTransformer;
  final bool Function(T item)? isSelected;
  final void Function(T item, int index)? onValueChange;

  final TextStyle? style;
  final bool disabel;
  final EdgeInsetsGeometry? padding;
  final double? spacing;
  final double? runSpacing;
  final TLWrapChoiceBoxType? type;

  @override
  _TLWrapChoiceBoxState<T> createState() => _TLWrapChoiceBoxState<T>();
}

GlobalKey<_TLWrapChoiceBoxState> _wrapChoiceBoxKey = GlobalKey<_TLWrapChoiceBoxState>();
GlobalKey<_TLWrapChoiceBoxState> get wrapChoiceBoxKey => _wrapChoiceBoxKey;

class _TLWrapChoiceBoxState<T> extends State<TLWrapChoiceBox<T>> {
  //
  int selectedIndex = -1;

  // 选择
  onSelectedChanged(int index) {
    setState(() {
      selectedIndex = index;
    });
  }

  // 迭代器生成List<Widget> 同步模式 sync*
  Iterable<Widget> get _inputSelects sync* {
    for (int index = 0; index < (widget.items?.length ?? 0); index++) {
      //
      Widget label = Text(
        widget.titleTransformer?.call(widget.items?[index] as T) ?? widget.items?[index] as String,
        softWrap: widget.type == TLWrapChoiceBoxType.wrap,
        maxLines: 4,
      );
      if (widget.type == TLWrapChoiceBoxType.center) {
        label = SizedBox(
          width: double.infinity,
          height: 40.h,
          child: Center(
            child: label,
          ),
        );
      } else if (widget.type == TLWrapChoiceBoxType.wrap) {
        label = Container(
          constraints: BoxConstraints(
            minHeight: 20.h,
          ),
          child: label,
        );
      } else if (widget.type == TLWrapChoiceBoxType.wrapSpaceBetween) {
        label = SizedBox(
          width: 82.w,
          height: 38.h,
          child: Center(
            child: label,
          ),
        );
      }
      //
      yield InputSelect(
        label: label,
        selected: widget.isSelected?.call(widget.items?[index] as T) ?? selectedIndex == index,
        onSelected: (bool value) {
          onSelectedChanged(index);
          widget.onValueChange?.call(widget.items?[index] as T, index);
        },
      );
    }
  }

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      child: AbsorbPointer(
        absorbing: widget.disabel,
        child: Container(
          width: double.infinity,
          padding: widget.padding ?? EdgeInsets.zero,
          child: Column(
            mainAxisAlignment: MainAxisAlignment.start,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              widget.title.isEmpty
                  ? const SizedBox.shrink()
                  : Padding(
                      padding: EdgeInsets.symmetric(
                        vertical: 8.h,
                      ),
                      child: Text(
                        widget.title,
                        style: widget.style ??
                            TextStyle(
                              color: Colors.black,
                              fontSize: 16.sp,
                              fontWeight: FontWeight.w500,
                            ),
                      ),
                    ),
              Wrap(
                spacing: widget.spacing ?? 8.w,
                runSpacing: widget.runSpacing ?? 8.h,
                children: _inputSelects.toList(),
              ),
            ],
          ),
        ),
      ),
    );
  }
}

// 实现 ChoiceInput
class InputSelect extends StatelessWidget {
  const InputSelect({
    Key? key,
    required this.label,
    required this.selected,
    required this.onSelected,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ChoiceChip(
      label: label,
      //选定的时候背景
      selectedColor: _kSelectedColor,
      backgroundColor: _kNormalColor,
      //被禁用得时候背景
      disabledColor: _kNormalColor,
      labelStyle: selected! ? _kSelectedTextStyle : _kNormalTextStyle,
      labelPadding: EdgeInsets.symmetric(horizontal: 8.w, vertical: 0),
      materialTapTargetSize: MaterialTapTargetSize.shrinkWrap,
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.circular(5.r),
      ),
      onSelected: onSelected,
      selected: selected!,
    );
  }

  final Widget label;
  final bool? selected;
  final ValueChanged<bool>? onSelected;
}

const Color _kSelectedColor = Color(0xFF1989FA);
const Color _kNormalColor = Color(0xFFF7F8FA);

TextStyle _kSelectedTextStyle = TextStyle(
  color: Colors.white,
  fontSize: 14.sp,
  fontWeight: FontWeight.w400,
);

TextStyle _kNormalTextStyle = TextStyle(
  color: Colors.black,
  fontSize: 14.sp,
  fontWeight: FontWeight.w400,
);